namespace FitConnect.Models;

public class FitConnectEnvironment {
    // List of Domains
    // https://wiki.fit-connect.fitko.dev/de/Betrieb/Dokumentation/Domains

    public static readonly FitConnectEnvironment Develop = new(
        "https://auth-dev.fit-connect.fitko.dev/token",
        new[] { "https://submission-api-dev.fit-connect.fitko.dev" },
        string.Empty, // "https://routing-api-testing.fit-connect.fitko.dev", // Dev does not have a routing API
        "https://portal.auth-dev.fit-connect.fitko.dev"
    ) { VerifiedKeysAreMandatory = false };

    public static readonly FitConnectEnvironment Testing = new(
        "https://auth-testing.fit-connect.fitko.dev/token",
        new[] { "https://submission-api-testing.fit-connect.fitko.dev" },
        "https://routing-api-testing.fit-connect.fitko.dev",
        "https://portal.auth-testing.fit-connect.fitko.dev"
    ) { VerifiedKeysAreMandatory = false };

    public static readonly FitConnectEnvironment Staging = new(
        "https://auth-refz.fit-connect.fitko.net/token",
        new[] { "submission-api-refz.fit-connect.niedersachsen.de" },
        string.Empty, // "https://routing-api-testing.fit-connect.fitko.dev", // Stage does not have a routing API
        "https://portal.auth-refz.fit-connect.fitko.net"
    );

    public static readonly FitConnectEnvironment Production = new(
        "https://auth-prod.fit-connect.fitko.net/token",
        new[] { "https://submission-api-prod.fit-connect.niedersachsen.de" },
        "https://routing-api-prod.fit-connect.fitko.net",
        "https://portal.auth-prod.fit-connect.fitko.net"
    );

    public FitConnectEnvironment(string sspUrl, string tokenUrl, string[] submissionUrl,
        string routingUrl) {
        SspUrl = sspUrl;
        TokenUrl = tokenUrl;
        SubmissionUrl = submissionUrl;
        RoutingUrl = routingUrl;
    }

    /// <summary>
    ///     Default constructor.
    /// </summary>
    /// <param name="tokenUrl">URL for receiving the OAuth token</param>
    /// <param name="submissionUrl">URL for the submission API</param>
    /// <param name="routingUrl">URL for the routing API</param>
    public FitConnectEnvironment(string tokenUrl, string[] submissionUrl, string routingUrl,
        string sspUrl) {
        SspUrl = sspUrl;
        TokenUrl = tokenUrl;
        SubmissionUrl = submissionUrl;
        RoutingUrl = routingUrl;
    }

    /// <summary>
    ///     Self service portal URL
    /// </summary>
    public string SspUrl { get; }

    /// <summary>
    ///     URL for receiving the OAuth token.
    /// </summary>
    public string TokenUrl { get; }

    /// <summary>
    ///     URL for the submission API.
    /// </summary>
    public string[] SubmissionUrl { get; }

    /// <summary>
    ///     URL for the routing API.
    /// </summary>
    public string RoutingUrl { get; }

    public bool VerifiedKeysAreMandatory { get; private init; } = true;

    /// <summary>
    ///     Creates the endpoints for the given environment.
    /// </summary>
    /// <param name="fitConnectEnvironment">Environment to get endpoints for</param>
    /// <returns></returns>
    /// <exception cref="ArgumentException">Not all environments are ready to use</exception>
    // public static FitConnectEnvironment Create(FitConnectEnvironment fitConnectEnvironment) {
    //     return fitConnectEnvironment switch {
    //         FitConnectEnvironment.Development => DevEnvironment,
    //         FitConnectEnvironment.Testing => throw new ArgumentException(
    //             "Not approved for online testing"),
    //         FitConnectEnvironment.Production => throw new ArgumentException("NOT PRODUCTION READY"),
    //         _ => throw new ArgumentOutOfRangeException(nameof(fitConnectEnvironment),
    //             fitConnectEnvironment, null)
    //     };
    // }
}
